package data.scripts;

import com.fs.starfarer.api.BaseModPlugin;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.impl.campaign.procgen.themes.SectorThemeGenerator;
import data.scripts.campaign.fleets.JunkPiratesExplorerFleetManager;
import data.scripts.campaign.fleets.SyndicateAspFleetManager;
import data.scripts.campaign.fleets.SyndicateAspHitSquadFleetManager;
import data.scripts.campaign.procgen.themes.JunkPiratesAnarchistThemeGenerator;
// Import every entry from your mod's data/world/generators.csv
import data.scripts.world.JunkGen;
//import data.scripts.omnifac.AddOmniFac;

import org.dark.shaders.util.ShaderLib;
import org.dark.shaders.light.LightData;
import org.dark.shaders.util.TextureData;
import exerelin.campaign.SectorManager;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import org.json.JSONException;
import org.json.JSONObject;

public class JunkPiratesModPlugin extends BaseModPlugin
{

    public static final boolean isExerelin;
    public static final boolean isIndEvo;
    
    public static boolean enableASP;
    public static boolean enableASPCourierFleets;
    public static boolean enableASPHitSquads;
    public static boolean enablePACK;
    public static boolean enablePACKDiplomats;
    public static boolean enableJunkPirates;
    public static boolean enableJunkExplorers;
    
    public static boolean enableJunkPiratesIntel;
    
    public static float junkPiratesFleetFrequencyModifier;
    public static float junkPiratesMaxFleetModifier;
    
//    
//    public static int minAnarchistConstellations;
//    public static int maxAnarchistConstellations;
//
//    public static int softMaxSpinerettes;
//
//    public static boolean enableProcGen;
//    public static boolean enableSpinerettes;
    
    static
    {
        boolean foundExerelin;
        try
        {
            Global.getSettings().getScriptClassLoader().loadClass("data.scripts.world.ExerelinGen");
            foundExerelin = true;
        }
        catch (ClassNotFoundException ex)
        {
            foundExerelin = false;
        }
        isExerelin = foundExerelin;
    }
    
    static
    {
        boolean foundIndEvo;
        try
        {
            Global.getSettings().getScriptClassLoader().loadClass("com.fs.starfarer.api.impl.campaign.ids.IndEvo_ids");
            foundIndEvo = true;
        }
        catch (ClassNotFoundException ex)
        {
            foundIndEvo = false;
        }
        isIndEvo = foundIndEvo;
    }
    
    
    
    private static void getProcGenSettings() {
    try {
        JSONObject settings = Global.getSettings().loadJSON("mendoncaModSettings.ini");

            enableASP = settings.getBoolean("enableASP");
            enableASPCourierFleets = settings.getBoolean("enableASPCourierFleets");
            enableASPHitSquads = settings.getBoolean("enableASPHitSquads");
            enablePACK = settings.getBoolean("enablePACK");
            enablePACKDiplomats = settings.getBoolean("enablePACKDiplomats");
            enableJunkPirates = settings.getBoolean("enableJunkPirates");
            enableJunkExplorers = settings.getBoolean("enableJunkExplorers");
            
            enableJunkPiratesIntel = settings.getBoolean("enableJunkPiratesIntel");
            
            junkPiratesFleetFrequencyModifier = (float) settings.getDouble("junkPiratesFleetFrequencyModifier");
            junkPiratesMaxFleetModifier = (float) settings.getDouble("junkPiratesMaxFleetModifier");
  
        } catch (IOException | JSONException ex) {
            System.out.println("JP Config Exception " + ex);
        }
    }
    
    private static void initJunkPirates() {
        new JunkGen().generate(Global.getSector());
    }
    @Override
    public void onNewGameAfterEconomyLoad() {
        if (isExerelin && !SectorManager.getCorvusMode())
            {
                addNexerelinRandommodeShips();
            }
        
        if (enableASPCourierFleets) {
            Global.getSector().addScript(new SyndicateAspFleetManager());
        }
        
        if (enableASPCourierFleets && enableASPHitSquads) {
            Global.getSector().getMemoryWithoutUpdate().set("$playerIsAspWanted", false);
            Global.getSector().addScript(new SyndicateAspHitSquadFleetManager());
        }
        
        if (enableJunkExplorers) {
            Global.getSector().addScript(new JunkPiratesExplorerFleetManager());
        }
        
        
//        if (enablePACKDiplomats) {
//            to do
//        }
//        if (enableJunkExplorers) {
//            to do
//        }
                         
    }
    @Override
    public void onNewGame() {
        if (isExerelin && !SectorManager.getCorvusMode())
            {
                return;
            }
        initJunkPirates();
    }
    
    public static void addNexerelinRandommodeShips() {
       
        List<String> aspships = new ArrayList<String>();
        
        aspships.add("syndicate_asp_hognose_p");
        aspships.add("syndicate_asp_kingcobra_p");
        aspships.add("syndicate_asp_diamondback_p");
        aspships.add("syndicate_asp_copperhead_p");
        aspships.add("syndicate_asp_gigantophis_p");
        
        FactionAPI faction = Global.getSector().getFaction("syndicate_asp");
        for (String id : aspships) {
            if (faction.knowsShip(id)) continue;
            faction.addKnownShip(id, true);
        }
    }


//    @Override
//    public void onNewGame()
//    {
//        // Calling a separate method avoids duplicate code with onEnabled()
//        initJunkPirates();
//    }

    @Override  
    public void onApplicationLoad()
    {  
    
        SectorThemeGenerator.generators.add(1, new JunkPiratesAnarchistThemeGenerator());
        
        getProcGenSettings();
        
        ShaderLib.init();  
        LightData.readLightDataCSV("data/lights/junk_pirates_light_data.csv");  
        TextureData.readTextureDataCSV("data/lights/junk_pirates_texture_data.csv");  
    }
}